/*
 * This file is part of Sharenet.
 * 
 * Sharenet is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 * 
 * You may obtain a copy of the license at:
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 * 
 * Copyright (c) 2024 Continuist <continuist02@gmail.com>
 */

use std::net::SocketAddr;
use std::sync::Arc;

use application::UseCase;
use axum::{
    extract::{Path, State},
    http::StatusCode,
    response::IntoResponse,
    routing::{delete, get, post, put},
    Json, Router,
};
use domain::{CreateProduct, CreateUser, Product, UpdateProduct, UpdateUser, User};
use tower_http::trace::TraceLayer;
use tower_http::cors::{CorsLayer, Any};
use tracing_subscriber::{layer::SubscriberExt, util::SubscriberInitExt, filter::EnvFilter};
use uuid::Uuid;

pub struct AppState<U, P> {
    user_service: Arc<U>,
    product_service: Arc<P>,
}

impl<U, P> Clone for AppState<U, P>
where
    U: Clone,
    P: Clone,
{
    fn clone(&self) -> Self {
        Self {
            user_service: self.user_service.clone(),
            product_service: self.product_service.clone(),
        }
    }
}

pub async fn run<U, P>(addr: SocketAddr, user_service: U, product_service: P)
where
    U: UseCase<User> + Clone + Send + Sync + 'static,
    P: UseCase<Product> + Clone + Send + Sync + 'static,
{
    tracing_subscriber::registry()
        .with(EnvFilter::new(
            std::env::var("RUST_LOG").unwrap_or_else(|_| "info".into()),
        ))
        .with(tracing_subscriber::fmt::layer())
        .init();

    let state = AppState {
        user_service: Arc::new(user_service),
        product_service: Arc::new(product_service),
    };

    // Configure CORS
    let cors = CorsLayer::new()
        .allow_origin(Any)
        .allow_methods(Any)
        .allow_headers(Any);

    let app = Router::new()
        .route("/users", post(create_user::<U>))
        .route("/users/:id", get(get_user::<U>))
        .route("/users", get(list_users::<U>))
        .route("/users/:id", put(update_user::<U>))
        .route("/users/:id", delete(delete_user::<U>))
        .route("/products", post(create_product::<P>))
        .route("/products/:id", get(get_product::<P>))
        .route("/products", get(list_products::<P>))
        .route("/products/:id", put(update_product::<P>))
        .route("/products/:id", delete(delete_product::<P>))
        .layer(cors)
        .layer(TraceLayer::new_for_http())
        .with_state(state);

    tracing::info!("listening on {}", addr);
    let listener = tokio::net::TcpListener::bind(addr).await.unwrap();
    axum::serve(listener, app).await.unwrap();
}

async fn create_user<U>(
    State(state): State<AppState<U, impl UseCase<Product>>>,
    Json(data): Json<CreateUser>,
) -> impl IntoResponse
where
    U: UseCase<User>,
{
    match state.user_service.create(data).await {
        Ok(user) => (StatusCode::CREATED, Json(user)).into_response(),
        Err(e) => (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response(),
    }
}

async fn get_user<U>(
    State(state): State<AppState<U, impl UseCase<Product>>>,
    Path(id): Path<Uuid>,
) -> impl IntoResponse
where
    U: UseCase<User>,
{
    match state.user_service.get(id).await {
        Ok(user) => (StatusCode::OK, Json(user)).into_response(),
        Err(e) => (StatusCode::NOT_FOUND, e.to_string()).into_response(),
    }
}

async fn list_users<U>(
    State(state): State<AppState<U, impl UseCase<Product>>>,
) -> impl IntoResponse
where
    U: UseCase<User>,
{
    match state.user_service.list().await {
        Ok(users) => (StatusCode::OK, Json(users)).into_response(),
        Err(e) => (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response(),
    }
}

async fn update_user<U>(
    State(state): State<AppState<U, impl UseCase<Product>>>,
    Path(id): Path<Uuid>,
    Json(data): Json<UpdateUser>,
) -> impl IntoResponse
where
    U: UseCase<User>,
{
    match state.user_service.update(id, data).await {
        Ok(user) => (StatusCode::OK, Json(user)).into_response(),
        Err(e) => (StatusCode::NOT_FOUND, e.to_string()).into_response(),
    }
}

async fn delete_user<U>(
    State(state): State<AppState<U, impl UseCase<Product>>>,
    Path(id): Path<Uuid>,
) -> impl IntoResponse
where
    U: UseCase<User>,
{
    match state.user_service.delete(id).await {
        Ok(_) => StatusCode::NO_CONTENT.into_response(),
        Err(e) => (StatusCode::NOT_FOUND, e.to_string()).into_response(),
    }
}

async fn create_product<P>(
    State(state): State<AppState<impl UseCase<User>, P>>,
    Json(data): Json<CreateProduct>,
) -> impl IntoResponse
where
    P: UseCase<Product>,
{
    match state.product_service.create(data).await {
        Ok(product) => (StatusCode::CREATED, Json(product)).into_response(),
        Err(e) => (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response(),
    }
}

async fn get_product<P>(
    State(state): State<AppState<impl UseCase<User>, P>>,
    Path(id): Path<Uuid>,
) -> impl IntoResponse
where
    P: UseCase<Product>,
{
    match state.product_service.get(id).await {
        Ok(product) => (StatusCode::OK, Json(product)).into_response(),
        Err(e) => (StatusCode::NOT_FOUND, e.to_string()).into_response(),
    }
}

async fn list_products<P>(
    State(state): State<AppState<impl UseCase<User>, P>>,
) -> impl IntoResponse
where
    P: UseCase<Product>,
{
    match state.product_service.list().await {
        Ok(products) => (StatusCode::OK, Json(products)).into_response(),
        Err(e) => (StatusCode::INTERNAL_SERVER_ERROR, e.to_string()).into_response(),
    }
}

async fn update_product<P>(
    State(state): State<AppState<impl UseCase<User>, P>>,
    Path(id): Path<Uuid>,
    Json(data): Json<UpdateProduct>,
) -> impl IntoResponse
where
    P: UseCase<Product>,
{
    match state.product_service.update(id, data).await {
        Ok(product) => (StatusCode::OK, Json(product)).into_response(),
        Err(e) => (StatusCode::NOT_FOUND, e.to_string()).into_response(),
    }
}

async fn delete_product<P>(
    State(state): State<AppState<impl UseCase<User>, P>>,
    Path(id): Path<Uuid>,
) -> impl IntoResponse
where
    P: UseCase<Product>,
{
    match state.product_service.delete(id).await {
        Ok(_) => StatusCode::NO_CONTENT.into_response(),
        Err(e) => (StatusCode::NOT_FOUND, e.to_string()).into_response(),
    }
}
