/*
 * This file is part of Sharenet.
 * 
 * Sharenet is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 * 
 * You may obtain a copy of the license at:
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 * 
 * Copyright (c) 2024 Continuist <continuist02@gmail.com>
 */

use anyhow::Result;
use clap::Parser;
use domain::{CreateProduct, CreateUser, Product, User, UpdateProduct, UpdateUser};
use application::UseCase;
use uuid::Uuid;

#[derive(Parser)]
#[command(author, version, about, long_about = None)]
pub struct Cli {
    #[command(subcommand)]
    command: Option<Commands>,
}

#[derive(Parser)]
pub enum Commands {
    /// User management commands
    User {
        #[command(subcommand)]
        command: UserCommands,
    },
    /// Product management commands
    Product {
        #[command(subcommand)]
        command: ProductCommands,
    },
}

#[derive(Parser)]
pub enum UserCommands {
    /// Create a new user
    Create {
        /// Username
        #[arg(short, long)]
        username: String,
        /// Email
        #[arg(short, long)]
        email: String,
    },
    /// List all users
    List,
    /// Get a user by ID
    Get {
        /// User ID
        #[arg(short, long)]
        id: Uuid,
    },
    /// Update a user
    Update {
        /// User ID
        #[arg(short, long)]
        id: Uuid,
        /// New username
        #[arg(short, long)]
        username: Option<String>,
        /// New email
        #[arg(short, long)]
        email: Option<String>,
    },
    /// Delete a user
    Delete {
        /// User ID
        #[arg(short, long)]
        id: Uuid,
    },
}

#[derive(Parser)]
pub enum ProductCommands {
    /// Create a new product
    Create {
        /// Product name
        #[arg(short, long)]
        name: String,
        /// Product description
        #[arg(short, long)]
        description: String,
    },
    /// List all products
    List,
    /// Get a product by ID
    Get {
        /// Product ID
        #[arg(short, long)]
        id: Uuid,
    },
    /// Update a product
    Update {
        /// Product ID
        #[arg(short, long)]
        id: Uuid,
        /// New name
        #[arg(short, long)]
        name: Option<String>,
        /// New description
        #[arg(short, long)]
        description: Option<String>,
    },
    /// Delete a product
    Delete {
        /// Product ID
        #[arg(short, long)]
        id: Uuid,
    },
}

impl Cli {
    pub async fn run<U, P>(self, user_service: U, product_service: P) -> Result<()>
    where
        U: UseCase<User>,
        P: UseCase<Product>,
    {
        match self.command {
            Some(Commands::User { command }) => match command {
                UserCommands::Create { username, email } => {
                    let user = user_service.create(CreateUser { username, email }).await?;
                    println!("Created user: {:?}", user);
                }
                UserCommands::List => {
                    let users = user_service.list().await?;
                    println!("Users: {:?}", users);
                }
                UserCommands::Get { id } => {
                    let user = user_service.get(id).await?;
                    println!("User: {:?}", user);
                }
                UserCommands::Update { id, username, email } => {
                    let update = UpdateUser { username, email };
                    let user = user_service.update(id, update).await?;
                    println!("Updated user: {:?}", user);
                }
                UserCommands::Delete { id } => {
                    user_service.delete(id).await?;
                    println!("Deleted user {}", id);
                }
            },
            Some(Commands::Product { command }) => match command {
                ProductCommands::Create { name, description } => {
                    let product = product_service.create(CreateProduct { name, description }).await?;
                    println!("Created product: {:?}", product);
                }
                ProductCommands::List => {
                    let products = product_service.list().await?;
                    println!("Products: {:?}", products);
                }
                ProductCommands::Get { id } => {
                    let product = product_service.get(id).await?;
                    println!("Product: {:?}", product);
                }
                ProductCommands::Update { id, name, description } => {
                    let update = UpdateProduct { name, description };
                    let product = product_service.update(id, update).await?;
                    println!("Updated product: {:?}", product);
                }
                ProductCommands::Delete { id } => {
                    product_service.delete(id).await?;
                    println!("Deleted product {}", id);
                }
            },
            None => {
                println!("No command provided. Use --help for usage information.");
            }
        }
        Ok(())
    }
}
