/*
 * This file is part of Sharenet.
 * 
 * Sharenet is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License.
 * 
 * You may obtain a copy of the license at:
 * https://creativecommons.org/licenses/by-nc-sa/4.0/
 * 
 * Copyright (c) 2024 Continuist <continuist02@gmail.com>
 */

use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};
use thiserror::Error;
use uuid::Uuid;
use std::future::Future;

#[derive(Debug, Error)]
pub enum DomainError {
    #[error("Entity not found: {0}")]
    NotFound(String),
    #[error("Invalid input: {0}")]
    InvalidInput(String),
    #[error("Internal error: {0}")]
    Internal(String),
}

unsafe impl Send for DomainError {}
unsafe impl Sync for DomainError {}

pub type Result<T> = std::result::Result<T, DomainError>;

pub trait Entity: Send + Sync {
    type Create: Send;
    type Update: Send;
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct User {
    pub id: Uuid,
    pub username: String,
    pub email: String,
    pub created_at: DateTime<Utc>,
    pub updated_at: DateTime<Utc>,
}

impl Entity for User {
    type Create = CreateUser;
    type Update = UpdateUser;
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Product {
    pub id: Uuid,
    pub name: String,
    pub description: String,
    pub created_at: DateTime<Utc>,
    pub updated_at: DateTime<Utc>,
}

impl Entity for Product {
    type Create = CreateProduct;
    type Update = UpdateProduct;
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct CreateUser {
    pub username: String,
    pub email: String,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct UpdateUser {
    pub username: Option<String>,
    pub email: Option<String>,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct CreateProduct {
    pub name: String,
    pub description: String,
}

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct UpdateProduct {
    pub name: Option<String>,
    pub description: Option<String>,
}

pub trait Repository<T, Create, Update>: Send + Sync 
where
    T: Send,
    Create: Send,
    Update: Send,
{
    fn create(&self, data: Create) -> impl Future<Output = Result<T>> + Send;
    fn find_by_id(&self, id: Uuid) -> impl Future<Output = Result<T>> + Send;
    fn find_all(&self) -> impl Future<Output = Result<Vec<T>>> + Send;
    fn update(&self, id: Uuid, data: Update) -> impl Future<Output = Result<T>> + Send;
    fn delete(&self, id: Uuid) -> impl Future<Output = Result<()>> + Send;
}

#[cfg(test)]
mod tests {
    use super::*;
    use chrono::Utc;

    mod user_tests {
        use super::*;

        #[test]
        fn test_user_entity_impl() {
            let user = User {
                id: Uuid::new_v4(),
                username: "test_user".to_string(),
                email: "test@example.com".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            assert_eq!(user.username, "test_user");
            assert_eq!(user.email, "test@example.com");
        }

        #[test]
        fn test_create_user_validation() {
            let create_user = CreateUser {
                username: "test_user".to_string(),
                email: "test@example.com".to_string(),
            };

            assert_eq!(create_user.username, "test_user");
            assert_eq!(create_user.email, "test@example.com");
        }

        #[test]
        fn test_update_user_partial() {
            let update_user = UpdateUser {
                username: Some("new_username".to_string()),
                email: None,
            };

            assert_eq!(update_user.username, Some("new_username".to_string()));
            assert_eq!(update_user.email, None);
        }
    }

    mod product_tests {
        use super::*;

        #[test]
        fn test_product_entity_impl() {
            let product = Product {
                id: Uuid::new_v4(),
                name: "Test Product".to_string(),
                description: "Test Description".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            assert_eq!(product.name, "Test Product");
            assert_eq!(product.description, "Test Description");
        }

        #[test]
        fn test_create_product_validation() {
            let create_product = CreateProduct {
                name: "Test Product".to_string(),
                description: "Test Description".to_string(),
            };

            assert_eq!(create_product.name, "Test Product");
            assert_eq!(create_product.description, "Test Description");
        }

        #[test]
        fn test_update_product_partial() {
            let update_product = UpdateProduct {
                name: Some("New Product Name".to_string()),
                description: None,
            };

            assert_eq!(update_product.name, Some("New Product Name".to_string()));
            assert_eq!(update_product.description, None);
        }
    }

    mod domain_error_tests {
        use super::*;

        #[test]
        fn test_not_found_error() {
            let error = DomainError::NotFound("User not found".to_string());
            assert_eq!(error.to_string(), "Entity not found: User not found");
        }

        #[test]
        fn test_invalid_input_error() {
            let error = DomainError::InvalidInput("Invalid email format".to_string());
            assert_eq!(error.to_string(), "Invalid input: Invalid email format");
        }

        #[test]
        fn test_internal_error() {
            let error = DomainError::Internal("Database connection failed".to_string());
            assert_eq!(error.to_string(), "Internal error: Database connection failed");
        }
    }

    mod repository_trait_tests {
        use super::*;
        use std::sync::Arc;
        use tokio::sync::RwLock;
        use std::collections::HashMap;

        // Mock implementation of Repository for testing
        struct MockRepository<T> {
            data: Arc<RwLock<HashMap<Uuid, T>>>,
        }

        impl<T: Clone + Send + Sync> MockRepository<T> {
            fn new() -> Self {
                Self {
                    data: Arc::new(RwLock::new(HashMap::new())),
                }
            }
        }

        #[tokio::test]
        async fn test_repository_create() {
            let repo = MockRepository::<User>::new();
            let create_user = CreateUser {
                username: "test_user".to_string(),
                email: "test@example.com".to_string(),
            };

            let user = User {
                id: Uuid::new_v4(),
                username: create_user.username.clone(),
                email: create_user.email.clone(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            repo.data.write().await.insert(user.id, user.clone());
            let guard = repo.data.read().await;
            let stored = guard.get(&user.id).unwrap();
            assert_eq!(stored.username, create_user.username);
            assert_eq!(stored.email, create_user.email);
        }

        #[tokio::test]
        async fn test_repository_find_by_id() {
            let repo = MockRepository::<User>::new();
            let user = User {
                id: Uuid::new_v4(),
                username: "test_user".to_string(),
                email: "test@example.com".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            repo.data.write().await.insert(user.id, user.clone());
            let guard = repo.data.read().await;
            let found = guard.get(&user.id).unwrap();
            assert_eq!(found.id, user.id);
        }

        #[tokio::test]
        async fn test_repository_find_all() {
            let repo = MockRepository::<User>::new();
            let user1 = User {
                id: Uuid::new_v4(),
                username: "user1".to_string(),
                email: "user1@example.com".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };
            let user2 = User {
                id: Uuid::new_v4(),
                username: "user2".to_string(),
                email: "user2@example.com".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            repo.data.write().await.insert(user1.id, user1.clone());
            repo.data.write().await.insert(user2.id, user2.clone());
            
            let all = repo.data.read().await;
            assert_eq!(all.len(), 2);
        }

        #[tokio::test]
        async fn test_repository_update() {
            let repo = MockRepository::<User>::new();
            let user = User {
                id: Uuid::new_v4(),
                username: "test_user".to_string(),
                email: "test@example.com".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            repo.data.write().await.insert(user.id, user.clone());
            let mut guard = repo.data.write().await;
            let stored = guard.get_mut(&user.id).unwrap();
            stored.username = "updated_user".to_string();
            
            drop(guard);
            let read_guard = repo.data.read().await;
            let updated = read_guard.get(&user.id).unwrap();
            assert_eq!(updated.username, "updated_user");
        }

        #[tokio::test]
        async fn test_repository_delete() {
            let repo = MockRepository::<User>::new();
            let user = User {
                id: Uuid::new_v4(),
                username: "test_user".to_string(),
                email: "test@example.com".to_string(),
                created_at: Utc::now(),
                updated_at: Utc::now(),
            };

            repo.data.write().await.insert(user.id, user.clone());
            repo.data.write().await.remove(&user.id);
            assert!(repo.data.read().await.get(&user.id).is_none());
        }
    }
}
